import os
import sys

from django import __version__ as django_version

from findmydevice import __version__


def main(argv):
    os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'findmydevice_project.settings.local')

    if '--version' not in argv:
        print(f'django-fmd v{__version__} (Django v{django_version})', file=sys.stderr)
        print(f'DJANGO_SETTINGS_MODULE={os.environ["DJANGO_SETTINGS_MODULE"]!r}', file=sys.stderr)

    try:
        from django.core.management import execute_from_command_line
    except ImportError as exc:
        raise ImportError(
            'Couldn\'t import Django. Are you sure it\'s installed and '
            'available on your PYTHONPATH environment variable? Did you '
            'forget to activate a virtual environment?'
        ) from exc
    try:
        execute_from_command_line(argv)
    except Exception as err:
        from bx_py_utils.error_handling import print_exc_plus

        print_exc_plus(err)
        raise


def start_test_server():
    """
    Entrypoint for "[tool.poetry.scripts]" script started by devshell command.
    """
    main(argv=[__file__, "run_testserver"] + sys.argv[1:])


if __name__ == '__main__':
    main(argv=sys.argv)



# Generated by Django 3.2.14 on 2022-07-11 22:08

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('findmydevice', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='location',
            name='raw_date',
            field=models.PositiveBigIntegerField(),
        ),
    ]



import logging

from django.http import HttpResponse
from django.views import View

from findmydevice.json_utils import parse_json
from findmydevice.models import Device
from findmydevice.services.device import get_device_by_token


logger = logging.getLogger(__name__)


class KeyView(View):
    """
    /key
    """

    def put(self, request):
        """
        The WebPage/Client requests the private key.
        """
        put_data = parse_json(request)
        access_token = put_data['IDT']

        # XXX: The fmd WebPage sends "Data":"0" why?
        index = put_data.get('Data')
        if index != '0':
            logger.warning('Get private key with Data!="0" ... Data is: %r', index)

        device: Device = get_device_by_token(token=access_token)
        privkey = device.privkey
        return HttpResponse(content_type='application/text', content=privkey)



import dataclasses

from django.test import SimpleTestCase

from findmydevice_project.tests.fixtures import get_fixtures
from fmd_client.data_classes import ClientDeviceData, LocationData


class ClientDataClassesTestCase(SimpleTestCase):
    def test_location(self):
        client_device_data = ClientDeviceData(
            plaintext_password='foo',
            hashed_password='06760f52af81bfe8f9406a65fda7f7b16484e91c55beaa36e5a1209985ffb0ee',
            encrypted_private_key=get_fixtures('device1/private_key'),
            public_key_base64_str=get_fixtures('device1/public_key'),
        )

        plain_location = LocationData(
            bat='66',
            lat='52.516265',
            lon='13.37738',
            provider='network',
            date=12346567890,
            encrypted=False,
        )

        encrypted_location = LocationData.encrypt(plain_location, client_device_data)
        assert encrypted_location.encrypted is True

        encrypted_keys = set()
        for key, value in dataclasses.asdict(encrypted_location).items():
            if key in ('date', 'encrypted'):
                continue
            assert isinstance(value, str)
            assert len(value) > 16
            encrypted_keys.add(key)
        assert encrypted_keys == {'bat', 'lat', 'lon', 'provider'}

        decrypted_location = LocationData.decrypt(encrypted_location, client_device_data)
        assert decrypted_location == plain_location



import logging

from django.http import HttpResponse, JsonResponse
from django.views import View

from findmydevice.json_utils import parse_json
from findmydevice.models import Device
from findmydevice.services.device import get_device_by_token


logger = logging.getLogger(__name__)


class DeviceView(View):
    """
    /device
    """

    def put(self, request):
        """
        Register a new Device
        """
        user_agent = request.headers.get('User-Agent')
        logger.info('New device register, user agent: %r', user_agent)

        data = parse_json(request)
        hashed_password = data['hashedPassword']

        # App sends hex digest in uppercase, the web page in lower case ;)
        hashed_password = hashed_password.lower()

        device = Device.objects.create(
            hashed_password=hashed_password,
            privkey=data['privkey'],
            pubkey=data['pubkey'],
            user_agent=user_agent,
        )
        access_token = {'DeviceId': device.short_id, 'AccessToken': ''}
        return JsonResponse(access_token)

    def post(self, request):
        """
        Delete a device
        """
        post_data = parse_json(request)

        access_token = post_data['IDT']
        device = get_device_by_token(token=access_token)
        logger.info('Delete device: %s', device)
        info = device.delete()
        logger.info('Delete info: %s', info)
        return HttpResponse(content=b'')



# flake8: noqa: E405, F403

import requests_mock

from findmydevice_project.settings.base import *


DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    }
}

SECRET_KEY = 'No individual secret... But this settings should only be used in tests ;)'

# Run the tests as on production: Without DBEUG:
DEBUG = False
TEMPLATE_DEBUG = False

ALLOWED_HOSTS = ('127.0.0.1', '0.0.0.0', 'localhost')


###################################################################################################
# Deny any not mocked requests
#
#
# def _unmocked_requests_error_message(request, response):
#     raise RuntimeError(
#         f'Unmocked request to {request.url} in tests, wrap with requests_mock.mock()!'
#     )
#
#
# fallback_mocker = requests_mock.Mocker()
# fallback_mocker.get(requests_mock.ANY, content=_unmocked_requests_error_message)
# fallback_mocker.__enter__()


###################################################################################################


LOGGING['formatters']['colored'][
    'format'
] = '%(log_color)s%(name)s %(levelname)8s %(cut_path)s:%(lineno)-3s %(message)s'



import datetime
import logging

from django.db import models

from findmydevice.models import Device
from findmydevice.models.base import FmdBaseModel


logger = logging.getLogger(__name__)


class Location(FmdBaseModel):
    device = models.ForeignKey(Device, on_delete=models.CASCADE)
    bat = models.CharField(max_length=256, unique=True)
    raw_date = models.PositiveBigIntegerField()
    lat = models.CharField(max_length=256, unique=True)
    lon = models.CharField(max_length=256, unique=True)
    provider = models.CharField(max_length=256, unique=True)

    def save(self, **kwargs):
        super().save(**kwargs)

        # Update "device.update_dt" field: So the admin change list will order the devices
        # by last location update ;)
        self.device.save(update_dt=True)

    @property
    def date_time(self) -> datetime.datetime:
        """
        Translates self.raw_date into a DateTime object
        """
        unix_time = self.raw_date / 1000
        dt = datetime.datetime.fromtimestamp(unix_time)
        logger.info('Raw date: %r -> %r -> %s', self.raw_date, unix_time, dt.isoformat())
        return dt

    def __str__(self):
        return f'Location {self.uuid} for {self.device} (Date: {self.raw_date}=={self.date_time})'

    def __repr__(self):
        return f'<{self.__str__()}>'

    class Meta:
        get_latest_by = ['create_dt']



import base64
import logging
import secrets
from hashlib import pbkdf2_hmac
from typing import Union

from Cryptodome.Cipher import AES, PKCS1_v1_5
from Cryptodome.PublicKey import RSA
from Cryptodome.Util.Padding import pad, unpad

from fmd_client.constants import FMD_BLOCK_SIZE, FMD_ITERATIONS, FMD_PASSWORD_SALT


logger = logging.getLogger(__name__)


def base64_encode(data: Union[bytes, str]) -> str:
    """
    >>> base64_encode(b'foo')
    'Zm9v'
    """
    base64_bytes = base64.b64encode(data)
    return base64_bytes.decode(encoding='utf-8')


def fmd_pbkdf2_hmac(raw_password: str, salt: bytes, iterations: int) -> bytes:
    key: bytes = pbkdf2_hmac(
        hash_name='sha1',
        password=bytes(raw_password, encoding='utf-8'),
        salt=salt,
        iterations=iterations,
        dklen=32,
    )
    return key


def generate_hashed_password(raw_password: str):
    """
    Generate a hashed password in the same way as the FMD web page and app.

    >>> generate_hashed_password(raw_password='foo')
    '06760f52af81bfe8f9406a65fda7f7b16484e91c55beaa36e5a1209985ffb0ee'

    https://gitlab.com/Nulide/findmydeviceserver/-/blob/master/web/logic.js
    """
    hashed_password_bytes = fmd_pbkdf2_hmac(
        raw_password,
        salt=FMD_PASSWORD_SALT,
        iterations=FMD_ITERATIONS * 2,  # == 3734
    )
    hashed_password_hex = hashed_password_bytes.hex()
    return hashed_password_hex


def decrypt_aes(*, cipher_text: str, raw_password: str) -> bytes:
    key_size = 256
    iv_size = 128

    iv_length = iv_size // 4  # == 32
    salt_length = key_size // 4  # == 64

    salt_hex = cipher_text[:salt_length]
    salt_bytes = bytes.fromhex(salt_hex)
    assert len(salt_bytes) == 32

    iv_hex = cipher_text[salt_length : (salt_length + iv_length)]
    iv_bytes = bytes.fromhex(iv_hex)
    assert len(iv_bytes) == 16

    encrypted_base64 = cipher_text[(salt_length + iv_length) :]
    ciphertext = base64.b64decode(encrypted_base64)

    hashed_password_bytes = fmd_pbkdf2_hmac(
        raw_password, salt=salt_bytes, iterations=FMD_ITERATIONS
    )

    aes_cipher = AES.new(key=hashed_password_bytes, mode=AES.MODE_CBC, iv=iv_bytes)
    plaintext_bytes = aes_cipher.decrypt(ciphertext)
    plaintext_bytes = unpad(plaintext_bytes, block_size=16)
    return plaintext_bytes


def encrypt_aes(*, plaintext_bytes: bytes, raw_password: str) -> str:
    key_size = 256
    iv_size = 128

    iv_nbytes = iv_size // 8  # == 16
    salt_nbytes = key_size // 8  # == 32

    salt_bytes = secrets.token_bytes(nbytes=salt_nbytes)
    assert len(salt_bytes) == 32

    iv_bytes = secrets.token_bytes(nbytes=iv_nbytes)
    assert len(iv_bytes) == 16

    hashed_password_bytes = fmd_pbkdf2_hmac(
        raw_password, salt=salt_bytes, iterations=FMD_ITERATIONS
    )

    aes_cipher = AES.new(key=hashed_password_bytes, mode=AES.MODE_CBC, iv=iv_bytes)

    plaintext_bytes_padded = pad(plaintext_bytes, block_size=FMD_BLOCK_SIZE, style='pkcs7')

    ciphertext = aes_cipher.encrypt(plaintext_bytes_padded)
    encrypted_base64 = base64_encode(ciphertext)
    cipher_text = salt_bytes.hex() + iv_bytes.hex() + encrypted_base64
    return cipher_text


def decrypt(*, data_base64: str, raw_private_key: str, raw_password: str) -> str:
    private_key = decrypt_aes(raw_password=raw_password, cipher_text=raw_private_key)

    rsa_key = RSA.import_key(private_key)
    rsa_cipher = PKCS1_v1_5.new(rsa_key)

    ciphertext = base64.b64decode(data_base64)

    decrypted_data_bytes = rsa_cipher.decrypt(ciphertext, sentinel='Decrypt Error')
    plaintext = decrypted_data_bytes.decode(encoding='utf-8')
    return plaintext


def encrypt(*, plaintext: str, raw_private_key: str, raw_password: str) -> str:
    private_key = decrypt_aes(raw_password=raw_password, cipher_text=raw_private_key)

    rsa_key = RSA.import_key(private_key)
    rsa_cipher = PKCS1_v1_5.new(rsa_key)

    data_bytes = bytes(plaintext, encoding='utf-8')
    decrypted_data_bytes = rsa_cipher.encrypt(data_bytes)
    decrypted_data_base64 = base64_encode(decrypted_data_bytes)
    return decrypted_data_base64


def generate_keys(raw_password: str) -> (str, str):
    logger.info('Generate new RSA keys')

    key = RSA.generate(2048)
    private_key = key.export_key()

    encrypted_private_key = encrypt_aes(
        plaintext_bytes=private_key + b'\n',
        raw_password=raw_password,
    )

    raw_public_key = key.publickey().export_key()
    public_key_base64_bytes = base64.b64encode(raw_public_key)
    public_key_base64_str = public_key_base64_bytes.decode(encoding='utf-8')

    return encrypted_private_key, public_key_base64_str



# Generated by Django 3.2.14 on 2022-07-19 17:04

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('findmydevice', '0006_push_commands'),
    ]

    operations = [
        migrations.AlterField(
            model_name='device',
            name='command2user',
            field=models.CharField(blank=True, editable=False, max_length=128, null=True),
        ),
        migrations.AlterField(
            model_name='device',
            name='hashed_password',
            field=models.CharField(editable=False, max_length=64),
        ),
        migrations.AlterField(
            model_name='device',
            name='privkey',
            field=models.CharField(editable=False, max_length=2048, unique=True),
        ),
        migrations.AlterField(
            model_name='device',
            name='pubkey',
            field=models.CharField(editable=False, max_length=512, unique=True),
        ),
    ]



import os

from django.core.management import call_command
from django.core.management.commands.runserver import Command as RunServerCommand


class Command(RunServerCommand):
    help = 'Run Django dev. Server'
    default_addr = '0.0.0.0'
    default_port = '8008'

    def verbose_call(self, command, **kwargs):
        self.stdout.write('\n')
        self.stdout.write('_' * 79)
        self.stdout.write(self.style.NOTICE(f' *** call "{command}" command with {kwargs}:'))
        self.stdout.write('\n')
        call_command(command, **kwargs)

    def handle(self, *args, **options):
        """
        INFO: The django reloader will call this multiple times!
        We check RUN_MAIN, that will be set in django.utils.autoreload
        So we can skip the first migrate run.
        """
        if os.environ.get('RUN_MAIN'):
            self.verbose_call('migrate', run_syncdb=True, interactive=False, verbosity=1)
            self.verbose_call('showmigrations', verbosity=1)

        if not options.get('addrport'):
            options['addrport'] = f'{self.default_addr}:{self.default_port}'

        self.verbose_call('runserver', **options)



from django.conf import settings


class NoAddPermissionsMixin:
    """
    User can add only if DEBUG is on
    """

    def has_add_permission(self, request):
        if settings.DEBUG:
            return super().has_add_permission(request)
        return False



import logging

from django.http import HttpResponseBadRequest, HttpResponseForbidden, JsonResponse
from django.views import View

from findmydevice.json_utils import parse_json
from findmydevice.models import Device
from findmydevice.services.device import new_access_token


logger = logging.getLogger(__name__)


class RequestAccessView(View):
    """
    /requestAccess
    """

    def put(self, request):  # TODO: Add lock here!
        """
        Response access token.
        """
        access_data = parse_json(request)

        hashed_password = access_data.get('HashedPassword') or access_data.get('Data')
        if not hashed_password:
            logger.error('No hashed password from "HashedPassword" or "Data" !')
            return HttpResponseBadRequest()

        # App sends hex digest in uppercase, the web page in lower case ;)
        hashed_password = hashed_password.lower()

        raw_short_id = access_data.get('DeviceId') or access_data.get('IDT')
        device = Device.objects.get_by_short_id(raw_short_id=raw_short_id)

        if hashed_password != device.hashed_password:
            logger.error(
                'Wrong password %r is not %r for %s',
                hashed_password,
                device.hashed_password,
                device,
            )
            return HttpResponseForbidden()

        access_token = new_access_token(device=device)
        access_token_reply = {'IDT': device.short_id, 'Data': access_token}
        return JsonResponse(access_token_reply)



# Generated by Django 3.2.14 on 2022-07-12 09:56

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('findmydevice', '0003_rename_deviceuser_device'),
    ]

    operations = [
        migrations.AddField(
            model_name='device',
            name='name',
            field=models.CharField(blank=True, help_text='Optional Name for this Device. e.g.: Username ;) Just displayed in the admin', max_length=64, null=True),
        ),
    ]



import json
import logging
from json import JSONDecodeError


logger = logging.getLogger(__name__)


def parse_json(request):
    content_type = request.content_type
    assert content_type == 'application/json', f'Wrong content type: {content_type!r}'
    encoding = request.encoding or 'utf-8'
    json_data = request.body.decode(encoding)
    try:
        data = json.loads(json_data)
    except JSONDecodeError as err:
        logger.error('JSON decode error: "%s" Data: %r', err, json_data)
        raise
    return data



import json
import logging

from bx_django_utils.humanize.pformat import pformat
from debug_toolbar.middleware import show_toolbar
from django.conf import settings
from django.http import HttpRequest, HttpResponse, JsonResponse

from findmydevice.json_utils import parse_json


logger = logging.getLogger(__name__)


def djdt_show(request):
    """
    Determining whether the Django Debug Toolbar should show or not.
    """
    if not settings.DEBUG:
        return False

    content_type = request.content_type
    if content_type == 'application/json':
        return False

    return show_toolbar(request)


class TracingMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request: HttpRequest):
        if settings.DEBUG:
            logger.debug('_' * 100)
            logger.debug(f' {request.method} {request.path}')
            content_type = request.content_type
            logger.debug(f'Content Type: {content_type}')
            logger.debug(f'Headers: {pformat(request.headers)}')
            if content_type == 'application/json':
                data = parse_json(request)
                logger.debug(pformat(data))
            else:
                logger.debug(f'Body: {request.body!r}')
            logger.debug(' -' * 50)

        response: HttpResponse = self.get_response(request)
        if settings.DEBUG:
            logger.debug(' -' * 50)
            respose_ok = 200 <= response.status_code <= 299
            if respose_ok:
                logger.debug(repr(response))
            else:
                logger.warning(repr(response))
            if respose_ok and hasattr(response, 'content'):
                content = response.content
                if isinstance(response, JsonResponse):
                    data = json.loads(content)
                    logger.debug(pformat(data))
                else:
                    if len(content) > 1000:
                        content = content[:1000] + b'...'
                    logger.debug(f'Content: {content!r}')

            logger.debug('-' * 100)

        return response



from django.http import HttpResponse
from django.views import View

import findmydevice


class VersionView(View):
    def get(self, request):
        return HttpResponse(
            ' | <a href="https://gitlab.com/jedie/django-find-my-device">'
            f'Django Find My Device v{findmydevice.__version__}</a> | '
        )



from django.conf import settings
from django.urls import path
from django.views.static import serve

import findmydevice
from findmydevice.views.base_views import PictureView
from findmydevice.views.command import CommandView
from findmydevice.views.device import DeviceView
from findmydevice.views.key import KeyView
from findmydevice.views.location import LocationView
from findmydevice.views.location_data_size import LocationDataSizeView
from findmydevice.views.push import PushView
from findmydevice.views.request_access import RequestAccessView
from findmydevice.views.version import VersionView
from findmydevice.views.web_page import FmdWebPageView


urlpatterns = [
    path('command', CommandView.as_view(), name='command'),
    path('location', LocationView.as_view(), name='location'),
    path('locationDataSize', LocationDataSizeView.as_view(), name='location_data_size'),
    path('picture', PictureView.as_view(), name='picture'),
    path('key', KeyView.as_view(), name='key'),
    path('device', DeviceView.as_view(), name='device'),
    path('push', PushView.as_view(), name='push'),
    path('requestAccess', RequestAccessView.as_view(), name='request_access'),
    path('version', VersionView.as_view(), name='version'),
    path('', FmdWebPageView.as_view(), name='fmd-web-page'),
]
if settings.DEBUG:
    # TODO: Serve from real Web server ;)
    urlpatterns.append(path('<path:path>', serve, {'document_root': findmydevice.WEB_PATH}))



# Generated by Django 3.2.14 on 2022-07-19 14:15

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('findmydevice', '0004_device_name'),
    ]

    operations = [
        migrations.AddField(
            model_name='device',
            name='user_agent',
            field=models.CharField(blank=True, editable=False, max_length=512, null=True),
        ),
        migrations.AddField(
            model_name='location',
            name='user_agent',
            field=models.CharField(blank=True, editable=False, max_length=512, null=True),
        ),
    ]



import logging
import secrets
from uuid import UUID

import requests
from django.conf import settings
from django.core import validators
from django.db import IntegrityError, models, transaction
from django.utils.translation import gettext_lazy as _

from findmydevice.exceptions import InvalidShortIdError
from findmydevice.models.base import FmdBaseModel


logger = logging.getLogger(__name__)

RANDOM_STRING_CHARS = 'bcdfghjklmnpqrstvwxyz0123456789'
SHORT_ID_LENGTH = 6


def get_short_id() -> str:
    short_id = ''.join(secrets.choice(RANDOM_STRING_CHARS) for _ in range(SHORT_ID_LENGTH))
    return short_id


class DeviceManager(models.Manager):
    def create(self, **kwargs):
        """
        Create a new object with a unique "short_id"
        """
        round = 0
        for round in range(1, settings.SHORT_ID_MAX_ROUNDS + 1):
            try:
                with transaction.atomic():
                    return super().create(short_id=get_short_id(), **kwargs)
            except IntegrityError as err:
                if 'short_id' not in str(err):
                    # Some other error happens
                    raise
                logger.warning(f'short_id collision, round: {round}')

        # If we really didn't find a unique ID, we have to increase SHORT_ID_LENGTH!
        raise RuntimeError(
            f'Can not find a unique "short_id" after {round} rounds!'
            f' (Please report this error to the project!)'
        )

    def get_by_short_id(self, raw_short_id):
        if not raw_short_id:
            logger.error('No "DeviceId" aka "IDT" aka "Short ID" !')
            raise InvalidShortIdError()  # -> BadRequest

        if len(raw_short_id) == 36:  # UUID / pk?
            # TODO: Remove in the future!
            try:
                uuid = UUID(raw_short_id)
            except ValueError as err:
                logger.error('Short ID %r is no UUID: %s', raw_short_id, err)
            else:
                device = Device.objects.filter(uuid=uuid).first()
                if device:
                    logger.warning('Found device by UUID: Will be removed in the future!')
                    return device

        max_length = Device._meta.get_field('short_id').max_length
        if len(raw_short_id) > max_length:
            logger.error(
                'Short ID %r length %i is more than max length: %i',
                raw_short_id,
                len(raw_short_id),
                max_length,
            )
            raise InvalidShortIdError()  # -> BadRequest

        device = Device.objects.filter(short_id=raw_short_id).first()
        if not device:
            logger.error('Device entry not found for: %r', raw_short_id)
            raise InvalidShortIdError()  # -> BadRequest
        else:
            return device


class Device(FmdBaseModel):
    """
    In FMD project it's named "user"
    """

    short_id = models.CharField(
        max_length=SHORT_ID_LENGTH,
        unique=True,
        db_index=True,
        editable=False,
        help_text=_('Device ID used for the App and Web page to identify this device'),
        validators=[validators.MinLengthValidator(SHORT_ID_LENGTH)],
    )
    name = models.CharField(
        max_length=64,
        null=True,
        blank=True,
        help_text=_('Optional Name for this Device. e.g.: Username ;) Just displayed in the admin'),
    )
    hashed_password = models.CharField(max_length=64, editable=False)
    privkey = models.CharField(max_length=4096, unique=True, editable=False)
    pubkey = models.CharField(max_length=1024, unique=True, editable=False)
    push_url = models.URLField(
        help_text=_('Push notification URL (Set by FMD app)'), blank=True, null=True
    )
    command2user = models.CharField(max_length=128, blank=True, null=True, editable=False)
    # pictures=

    objects = DeviceManager()

    def push_notification(self, text):
        if not self.push_url:
            logger.error('No push URL registered for %s', self)
        else:
            data = text.encode(encoding='utf-8')
            requests.post(self.push_url, data=data)

    def __str__(self):
        name = self.name or '>no name<'
        return f'{name} ({self.short_id or self.uuid})'



# Generated by Django 3.2.14 on 2022-07-12 09:33

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('findmydevice', '0002_alter_location_raw_date'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='DeviceUser',
            new_name='Device',
        ),
        migrations.RenameField(
            model_name='location',
            old_name='device_user',
            new_name='device',
        ),
    ]



# Generated by Django 3.2.14 on 2022-07-31 15:41

import django.core.validators
from django.db import migrations, models

from findmydevice.models.device import get_short_id


class Migration(migrations.Migration):

    dependencies = [
        ('findmydevice', '0007_editable'),
    ]

    operations = [
        migrations.AddField(
            model_name='device',
            name='short_id',
            field=models.CharField(
                db_index=True,
                default=get_short_id,  # can collide in very, very rare cases ;)
                editable=False,
                help_text='Device ID used for the App and Web page to identify this device',
                max_length=6,
                unique=True,
                validators=[django.core.validators.MinLengthValidator(6)],
            ),
            preserve_default=False,
        ),
    ]



# Generated by Django 3.2.14 on 2022-07-19 16:29

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('findmydevice', '0005_user_agent'),
    ]

    operations = [
        migrations.AddField(
            model_name='device',
            name='command2user',
            field=models.CharField(blank=True, max_length=128, null=True),
        ),
        migrations.AddField(
            model_name='device',
            name='push_url',
            field=models.URLField(
                blank=True, help_text='Push notification URL (Set by FMD app)', null=True
            ),
        ),
    ]



import datetime
from unittest import mock
from uuid import UUID

from bx_django_utils.test_utils.datetime import MockDatetimeGenerator
from bx_django_utils.test_utils.html_assertion import (
    HtmlAssertionMixin,
    assert_html_response_snapshot,
)
from django.contrib.auth.models import User
from django.template.defaulttags import CsrfTokenNode
from django.test import TestCase, override_settings
from django.utils import timezone
from model_bakery import baker

from findmydevice import __version__
from findmydevice.models import Device, Location


class AdminAnonymousTests(TestCase):
    """
    Anonymous will be redirected to the login page.
    """

    def test_login_en(self):
        response = self.client.get('/admin/', secure=True, HTTP_ACCEPT_LANGUAGE='en')
        self.assertRedirects(
            response, expected_url='/admin/login/?next=/admin/', fetch_redirect_response=False
        )

    def test_login_de(self):
        response = self.client.get('/admin/', secure=True, HTTP_ACCEPT_LANGUAGE='de')
        self.assertRedirects(
            response, expected_url='/admin/login/?next=/admin/', fetch_redirect_response=False
        )


@override_settings(SECURE_SSL_REDIRECT=False)
class AdminUserTests(HtmlAssertionMixin, TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.superuser = baker.make(User, is_staff=True, is_active=True, is_superuser=True)

    def test_superuser_access(self):
        self.client.force_login(self.superuser)

        with mock.patch.object(CsrfTokenNode, 'render', return_value='MockedCsrfTokenNode'):
            response = self.client.get('/admin/')
        self.assertTemplateUsed(response, 'admin/index.html')

        self.assert_html_parts(
            response,
            parts=(
                '<title>Site administration | Find My Device admin</title>',
                (
                    '<h1 id="site-name">'
                    f'<a href="/admin/">Django Find My Device v{__version__}</a>'
                    '</h1>'
                ),
                '<a href="/admin/findmydevice/device/">Devices</a>',
                '<a href="/admin/findmydevice/location/">Locations</a>',
            ),
        )
        assert_html_response_snapshot(response, validate=False)

    @mock.patch.object(timezone, 'now', MockDatetimeGenerator(datetime.timedelta(minutes=1)))
    def test_superuser_device(self):
        device = baker.make(Device, uuid=UUID(int=1), short_id='xyz012', name='Smartphone John')
        baker.make(Location, device=device)

        self.client.force_login(self.superuser)

        with mock.patch.object(CsrfTokenNode, 'render', return_value='MockedCsrfTokenNode'):
            response = self.client.get('/admin/findmydevice/device/')
        self.assertTemplateUsed(response, 'admin/change_list.html')

        self.assert_html_parts(
            response,
            parts=('<title>Select device to change | Find My Device admin</title>',),
        )
        assert_html_response_snapshot(response, validate=False)

    def test_superuser_location(self):
        self.client.force_login(self.superuser)

        with mock.patch.object(CsrfTokenNode, 'render', return_value='MockedCsrfTokenNode'):
            response = self.client.get('/admin/findmydevice/location/')
        self.assertTemplateUsed(response, 'admin/change_list.html')

        self.assert_html_parts(
            response,
            parts=('<title>Select location to change | Find My Device admin</title>',),
        )
        assert_html_response_snapshot(response, validate=False)



# Generated by Django 3.2.15 on 2022-08-06 17:56

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('findmydevice', '0008_device_short_id'),
    ]

    operations = [
        migrations.AlterField(
            model_name='device',
            name='privkey',
            field=models.CharField(editable=False, max_length=4096, unique=True),
        ),
        migrations.AlterField(
            model_name='device',
            name='pubkey',
            field=models.CharField(editable=False, max_length=1024, unique=True),
        ),
    ]
